==> vQMgArq1o4U1-0.0.45.tgz: package/sandbox.test.js <==
// Run one test with -g + keyword:
// > mocha *.test.js -g KEYWORD
// for debug
// > npm run debug -- -g context
// >export  DEBUG=mocha,cluster,client,worker,sandbox && mocha *.test.js -g KEYWORD

// -----------------------------------------------------------------------------
// Init
// -----------------------------------------------------------------------------

const should = require('should')
const sandbox = require('./sandbox.js')
const debug = require('debug')('sandbox:tests-sandbox')

// -----------------------------------------------------------------------------
// Tests
// -----------------------------------------------------------------------------

describe('Sandbox', function () {
  it('Loads', function () {
    // import './sandbox.js'
    sandbox('', null, null)
    var a = '2'
    a.should.eql('2')
  })

  it('should run simple addition', function (done) {
    sandbox('RESULT = 1 + 1;', null, null, function (err, res) {
      if (err) {
        debug('err', err)
      } else {
        debug('res', res)
      }
      res.result.should.eql(2)
      done()
    })
  })

  it('should run simple assignment', function (done) {
    sandbox('a=1;RESULT = 1+a', null, null, function (err, res) {
      if (err) {
        debug('err', err)
      } else {
        debug('res', res)
      }
      res.result.should.eql(2)
      done()
    })
  })

  it('should run multiline text', function (done) {
    sandbox('a=1 \n RESULT = 1+a', null, null, function (err, res) {
      if (err) {
        debug('err', err)
      } else {
        debug('res', res)
      }
      res.result.should.eql(2)
      done()
    })
  })

  it('should recognize objects', function (done) {
    sandbox('obj = {a:1} \n RESULT = 1+obj.a', null, null, function (err, res) {
      if (err) {
        debug('err', err)
      } else {
        debug('res', res)
      }
      res.result.should.eql(2)
      done()
    })
  })

  it('handles mysterious issue with literals', function (done) {
    var myCode = 'RESULT = "f"'

    sandbox(myCode, null, null, function (err, res) {
      debug('res', res)
      debug('err', err)

      res.result.should.eql('f')
      done()
    })
  })

  it('should gracefully handle syntax errors', function (done) {
    sandbox('hi )there', null, null, function (err, res) {
      debug('err', err)
      err.result.should.eql('SyntaxError : unknown: Unexpected token, expected ; (1:3)')
      done()
    })
  })

  it('should effectively prevent code from accessing node.js ', function (done) {
    sandbox('RESULT = process.platform', null, null, function (err, res) {
      err.result.should.eql('ReferenceError : process is not defined')
      done()
    })
  })

  it('should effectively prevent code from circumventing the sandbox', function (done) {
    sandbox("var sys=require('sys'); sys.puts('Up in your fridge')", null, null, function (err, res) {
      // debug('res', res)
      err.result.should.eql('ReferenceError : require is not defined')
      done()
    })
  })

  it('sends console logs to sandbox parent on success', function (done) {
    var myCode = 'console.log("hello" ); RESULT=1'

    sandbox(myCode, null, null, function (err, res) {
      debug('res', res)
      debug('err', err)

      res.console[0].should.eql('hello\n')
      done()
    })
  })

  it('sends console logs to sandbox parent, on failure', function (done) {
    var myCode = 'console.log("hello" ); undefinedFunctionFails();  RESULT=1'

    sandbox(myCode, null, null, function (err, res) {
      debug('res', res)
      debug('err', err)

      err.console[0].should.eql('hello\n')
      done()
    })
  })

  it('can load whitelisted libraries', function (done) {
    var myCode = 'fs.readFileSync("package.json") ; RESULT = 42'
    var myLibraries = [{namespace: 'fs', npm: 'fs'}]

    sandbox(myCode, myLibraries, null, function (err, res) {
      if (err) {
        debug('err', err)
      } else {
        debug('res', res)
      }
      res.result.should.eql(42)
      done()
    })
  })

  it('can load MULTIPLE white-listed libraries in the default context', function (done) {
    var myCode = 'fs.readFileSync("package.json") ; RESULT = util.isArray([])'
    var myLibraries = [
      { namespace: 'fs', npm: 'fs' },
      { namespace: 'util', npm: 'util' }
    ]

    sandbox(myCode, myLibraries, null, function (err, res) {
      debug(err)
      debug('res', res)
      res.result.should.eql(true)
      done()
    })
  })

  it('will not execute a dependency NOT in the default context', function (done) {
    // sb.run(' util.isNumber(false) ', function(output) {
    //   output.result.should.eql('\'ReferenceError: util is not defined\'');
    //   done();
    // });
    var myCode = 'fs.readFileSync("package.json") '
    var myLibraries = [{ namespace: 'util', npm: 'util' }] // fs not here

    sandbox(myCode, myLibraries, null, function (err, res) {
      debug('err', err)
      err.result.should.eql('ReferenceError : fs is not defined')
      done()
    })
  })

  it('can append an object context to the defaultContext ', function (done) {
    sandbox('RESULT = test', null, {test: 1234}, function (err, res) {
      if (err) {
        debug('err', err)
      } else {
        debug('res', res)
      }
      res.result.should.eql(1234)
      done()
    })
  })

  it('can append a function to defaultContext ', function (done) {
    sandbox('RESULT = test()', null, { test: function () { return 1234 } }, function (err, res) {
      debug('err', err)
      debug('res', res)

      res.result.should.eql(1234)
      done()
    })
  })

  // can wrap functions, so that in wireline the sandbox runner can wrap the functions so that only the RESULT variable is exposed in the sandbox
  it('can wrap functions', function (done) {
    var script = `function wrapped(){
      function add(first, second){
        return first + second;
      }
      return add(20, 22)
    }
     RESULT = wrapped()`

    sandbox(script, null, {}, function (err, res) {
      debug('err', err)
      debug('res', res)

      res.result.should.eql(42)
      done()
    })
  })

  it('can have wrapped functions access INPUT object', function (done) {
    var script = `function wrapped(){
      function add(first, second){
        return first + Number(INPUT.a);
      }
      return add(20, 22)
    }
     RESULT = wrapped()`

    sandbox(script, null, { INPUT: { a: 22 } }, function (err, res) {
      debug('err', err)
      debug('res', res)

      res.result.should.eql(42)
      done()
    })
  })

  // it('TODO: can have wrapped function using "done()" ', function(done) {
  //   // var myCode = "x=1;rp('http://www.google.com').then( function(){x=2 \n callback() })"
  //   var myCode = "function botFunction(){\
  //     a=1; setTimeout(function(){\
  //       a=2;\
  //       return a\
  //     }, 10);\
  //     return a\
  //   };\
  //     RESULT = botFunction() "
  //
  //   myLibraries = []
  //   sandbox( myCode , myLibraries, null, function(err, res ){
  //     debug('res: ', res);
  //     res.result.should.eql(2);
  //     done();
  //   })
  // });

  it('done() async api works with long-running async functions', function (done) {
    var myCode = 'RESULT=1; setTimeout(function () { RESULT=2 \n done() }, 100)'
    // var myCode = "a=3 \n RESULT = 1+a \n done()"

    var myLibraries = [{ namespace: 't', npm: 'timeout' }]
    sandbox(myCode, myLibraries, null, function (err, res) {
      if (err) {
        debug('err', err)
      } else {
        debug('res', res)
      }
      res.result.should.eql(2)
      done()
    })
  })

  it('done() can return a value async', function (done) {
    var myCode = 'RESULT=1; setTimeout(function(){ done( 2 ) }, 100)'
    // var myCode = "a=3 \n RESULT = 1+a \n done()"

    var myLibraries = [{ namespace: 't', npm: 'timeout' }]
    sandbox(myCode, myLibraries, null, function (err, res) {
      if (err) {
        debug('err', err)
      } else {
        debug('res', res)
      }
      res.result.should.eql(2)
      done()
    })
  })

  // it('should run async libraries like request-promise', function(done) {
  //   var myCode = `rp('http://mockbin.org/bin/40bd780c-a6ae-456c-8dbd-d61d9f47ba3b')
  //   .then( function ( htmlBody ) {
  //     debug( 'htmlBody', htmlBody);
  //     done(htmlBody)
  //   }).catch( function(err) {
  //     debug('err', err )
  //   })`
  //
  //   myLibraries = [{ namespace: 't', npm: 'timeout'}]
  //
  //   sandbox( myCode , myLibraries, null, function(err, res ){
  //     result = JSON.parse( res.result )
  //     // debug('res', res);
  //     result.foo.should.eql('hello world');
  //     done();
  //   })
  // });

  it('should run async libraries like timeout', function (done) {
    var myCode = `
      t.timeout(1000, function(){
        done( { a: 1})
      })
    `

    var myLibraries = [{ namespace: 't', npm: 'timeout' }]

    sandbox(myCode, myLibraries, null, function (err, res) {
      if (err) {
        debug('err', err)
      } else {
        debug('res', res)
      }

      res.result.a.should.eql(1)
      done()
    })
  })

  // it('uses in-house Request promise', function(done) {
  //   // myLibraries = [{ namespace: 't', npm: 'timeout'}]
  //
  //   var myCode = `httpRequest({ url: 'http://mockbin.org/bin/40bd780c-a6ae-456c-8dbd-d61d9f47ba3b' })
  //   .then( function ( htmlBody ) {
  //     console.log( 'htmlBody', htmlBody);
  //     done(htmlBody)
  //
  //   }).catch( function(err) {
  //     console.log('err', err )
  //   })`
  //
  //   sandbox( myCode , null, null, function(err, res ){
  //     debug('err', err);
  //     debug('res', res.result)
  //     result = JSON.parse( res.result )
  //     // debug('res', res);
  //     result.foo.should.eql('hello world');
  //     done();
  //   })
  // });

  it('uses in-house request library function externally', function (done) {
    // for individual testt
    const requestlib = require('request')

    var httpRequestFunction = function (requestObj) {
      return new Promise(function (resolve, reject) {
        requestlib(requestObj, function (err, IncomingMessage, body) {
          if (err) {
              // console.log('httpRequest err', err);
            return reject(err)
          }
          resolve(body)
        })
      })
    }

    var myCode = `httpRequestFunction({ url: 'http://mockbin.org/bin/40bd780c-a6ae-456c-8dbd-d61d9f47ba3b' })
      .then( function ( htmlBody ) {
        console.log( 'htmlBody', htmlBody);
        done(htmlBody)

      }).catch( function(err) {
        console.log('err', err )
      })`

    sandbox(myCode, null, { httpRequestFunction: httpRequestFunction }, function (err, res) {
      if (err) debug('err', err); else debug('res', res.result)

      var result = JSON.parse(res.result)
      // debug('res', res);
      result.foo.should.eql('hello world')
      done()
    })
  })

  it('uses native ES6 Promise in sandbox', function (done) {
    // myLibraries = [{ namespace: 't', npm: 'timeout'}]
    var debugPromise = function (requestObj) {
      return new Promise(function (resolve, reject) {
        resolve({ a: 1 })
      })
    }

    var myCode = `debugPromise()
    .then( function ( res ) {
      console.log( 'res', res);
      done(res)

    }).catch( function(err) {
      console.log('err', err )
    })`

    sandbox(myCode, null, { debugPromise: debugPromise }, function (err, res) {
      if (err) {
        debug('err', err)
      } else {
        debug('res', res)
      }

      res.result.a.should.eql(1)
      done()
    })
  })

  // TODO:
  it('Unknown errors reported ', function (done) {
    var myCode = 'function test(){ return INPUT[0] }; RESULT = test()'

    sandbox(myCode, null, { INPUT: { a: 22 } }, function (err, res) {
      debug('res', res)
      debug('err', err)

      // result = JSON.parse( res.result )
      // debug('parsed res', result);
      res.status.should.eql('COMPLETE')
      done()
    })
  })

  it('Allows variable assignment within Request', function (done) {
    // for individual testt
    const requestlib = require('request')

    var httpRequestFunction = function (requestObj) {
      return new Promise(function (resolve, reject) {
        requestlib(requestObj, function (err, IncomingMessage, body) {
          if (err) {
              // console.log('httpRequest err', err);
            return reject(err)
          }
          resolve(body)
        })
      })
    }

    var myCode = `var assignment = 'useless';
    httpRequestFunction({ url: 'http://mockbin.org/bin/40bd780c-a6ae-456c-8dbd-d61d9f47ba3b' })
    .then(function( body ){
      done( body );
    })`

    sandbox(myCode, null, { httpRequestFunction: httpRequestFunction }, function (err, res) {
      if (err) {
        debug('err', err)
      } else {
        debug('res', res)
      }

      var result = JSON.parse(res.result)
      debug('parsed res', result)
      result.foo.should.eql('hello world')
      done()
    })
  })

  it('returns undefined library correctly', function (done) {
    var script = `function wrapped(){
        return request({ url: "http://mockbin.org/bin/17889536-56e3-498d-9caa-be279a7e5a8d/" })
        .then(function( body ){ RESULT = body; done()})
      };
      wrapped()`

    sandbox(script, null, { INPUT: { a: 22 } }, function (err, res) {
      debug('sandbox2test.js - err', err)
      debug('sandbox2test.js - res', res)
      err.status.should.eql('ERROR')
      done()
    })
  })

  it('TODO: issue identifying async done vs return', function (done) {
    this.skip()
    var myCode = `
      RESULT=1
      if (false) {
        setTimeout(function () {
           RESULT=2
           done()
         }, 100)
      } else {
        RESULT=2
      }`
      // var myCode = "a=3 \n RESULT = 1+a \n done()"

    myLibraries = [{ namespace: 't', npm: 'timeout'}]
    sandbox(myCode, myLibraries, null, function (err, res) {
      res.result.should.eql(2)
      done()
    })
  })

  it('uses native ES6 async/await syntax', function (done) {
    this.timeout(5 * 1000)
    // myLibraries = [{ namespace: 't', npm: 'timeout'}]
    var debugPromise = function (requestObj) {
      return new Promise(function (resolve, reject) {
        resolve({ a: 1 })
      })
    }

    var myCode = `
    async function test () {
      var a = await debugPromise()
      console.log(a)
      done(a)
      return a
    }
    test()
    `

    sandbox(myCode, null, { debugPromise: debugPromise }, function (err, res) {
      if (err) {
        debug('err', err)
      } else {
        debug('res', res)
      }

      res.result.a.should.eql(1)
      done()
    })
  })

  it('uses native ES6 let const', function (done) {
    // myLibraries = [{ namespace: 't', npm: 'timeout'}]
    var debugPromise = function (requestObj) {
      return new Promise(function (resolve, reject) {
        resolve({ a: 1 })
      })
    }

    var myCode = `
      function test () {
        let a = {a: 1}
        done(a)
      }
      test()
    `

    sandbox(myCode, null, { debugPromise: debugPromise }, function (err, res) {
      if (err) {
        debug('err', err)
      } else {
        debug('res', res)
      }

      res.result.a.should.eql(1)
      done()
    })
  })
})

==> vQMgArq1o4U1-0.0.48.tgz: package/sandbox.test.js <==
// Run one test with -g + keyword:
// > mocha *.test.js -g KEYWORD
// for debug
// > npm run debug -- -g context
// >export  DEBUG=mocha,cluster,client,worker,sandbox && mocha *.test.js -g KEYWORD

// -----------------------------------------------------------------------------
// Init
// -----------------------------------------------------------------------------

const should = require('should')
const sandbox = require('./sandbox.js')
const debug = require('debug')('sandbox:tests-sandbox')

// -----------------------------------------------------------------------------
// Tests
// -----------------------------------------------------------------------------

describe('Sandbox', function () {
  it('Loads', function () {
    // import './sandbox.js'
    sandbox('', null, null)
    var a = '2'
    a.should.eql('2')
  })

  it('should run simple addition', function (done) {
    sandbox('RESULT = 1 + 1;', null, null, function (err, res) {
      if (err) {
        debug('err', err)
      } else {
        debug('res', res)
      }
      res.result.should.eql(2)
      done()
    })
  })

  it('should run simple assignment', function (done) {
    sandbox('a=1;RESULT = 1+a', null, null, function (err, res) {
      if (err) {
        debug('err', err)
      } else {
        debug('res', res)
      }
      res.result.should.eql(2)
      done()
    })
  })

  it('should run multiline text', function (done) {
    sandbox('a=1 \n RESULT = 1+a', null, null, function (err, res) {
      if (err) {
        debug('err', err)
      } else {
        debug('res', res)
      }
      res.result.should.eql(2)
      done()
    })
  })

  it('should recognize objects', function (done) {
    sandbox('obj = {a:1} \n RESULT = 1+obj.a', null, null, function (err, res) {
      if (err) {
        debug('err', err)
      } else {
        debug('res', res)
      }
      res.result.should.eql(2)
      done()
    })
  })

  it('handles mysterious issue with literals', function (done) {
    var myCode = 'RESULT = "f"'

    sandbox(myCode, null, null, function (err, res) {
      debug('res', res)
      debug('err', err)

      res.result.should.eql('f')
      done()
    })
  })

  it('should gracefully handle syntax errors', function (done) {
    sandbox('hi )there', null, null, function (err, res) {
      debug('err', err)
      err.result.should.eql('SyntaxError : Unexpected token )')
      done()
    })
  })

  it('should effectively prevent code from accessing node.js ', function (done) {
    sandbox('RESULT = process.platform', null, null, function (err, res) {
      err.result.should.eql('ReferenceError : process is not defined')
      done()
    })
  })

  it('should effectively prevent code from circumventing the sandbox', function (done) {
    sandbox("var sys=require('sys'); sys.puts('Up in your fridge')", null, null, function (err, res) {
      // debug('res', res)
      err.result.should.eql('ReferenceError : require is not defined')
      done()
    })
  })

  it('sends console logs to sandbox parent on success', function (done) {
    var myCode = 'console.log("hello" ); RESULT=1'

    sandbox(myCode, null, null, function (err, res) {
      debug('res', res)
      debug('err', err)

      res.console[0].should.eql('hello\n')
      done()
    })
  })

  it('sends console logs to sandbox parent, on failure', function (done) {
    var myCode = 'console.log("hello" ); undefinedFunctionFails();  RESULT=1'

    sandbox(myCode, null, null, function (err, res) {
      debug('res', res)
      debug('err', err)

      err.console[0].should.eql('hello\n')
      done()
    })
  })

  it('can load whitelisted libraries', function (done) {
    var myCode = 'fs.readFileSync("package.json") ; RESULT = 42'
    var myLibraries = [{namespace: 'fs', npm: 'fs'}]

    sandbox(myCode, myLibraries, null, function (err, res) {
      if (err) {
        debug('err', err)
      } else {
        debug('res', res)
      }
      res.result.should.eql(42)
      done()
    })
  })

  it('can load MULTIPLE white-listed libraries in the default context', function (done) {
    var myCode = 'fs.readFileSync("package.json") ; RESULT = util.isArray([])'
    var myLibraries = [
      { namespace: 'fs', npm: 'fs' },
      { namespace: 'util', npm: 'util' }
    ]

    sandbox(myCode, myLibraries, null, function (err, res) {
      debug(err)
      debug('res', res)
      res.result.should.eql(true)
      done()
    })
  })

  it('will not execute a dependency NOT in the default context', function (done) {
    // sb.run(' util.isNumber(false) ', function(output) {
    //   output.result.should.eql('\'ReferenceError: util is not defined\'');
    //   done();
    // });
    var myCode = 'fs.readFileSync("package.json") '
    var myLibraries = [{ namespace: 'util', npm: 'util' }] // fs not here

    sandbox(myCode, myLibraries, null, function (err, res) {
      debug('err', err)
      err.result.should.eql('ReferenceError : fs is not defined')
      done()
    })
  })

  it('can append an object context to the defaultContext ', function (done) {
    sandbox('RESULT = test', null, {test: 1234}, function (err, res) {
      if (err) {
        debug('err', err)
      } else {
        debug('res', res)
      }
      res.result.should.eql(1234)
      done()
    })
  })

  it('can append a function to defaultContext ', function (done) {
    sandbox('RESULT = test()', null, { test: function () { return 1234 } }, function (err, res) {
      debug('err', err)
      debug('res', res)

      res.result.should.eql(1234)
      done()
    })
  })

  // can wrap functions, so that in wireline the sandbox runner can wrap the functions so that only the RESULT variable is exposed in the sandbox
  it('can wrap functions', function (done) {
    var script = `function wrapped(){
      function add(first, second){
        return first + second;
      }
      return add(20, 22)
    }
     RESULT = wrapped()`

    sandbox(script, null, {}, function (err, res) {
      debug('err', err)
      debug('res', res)

      res.result.should.eql(42)
      done()
    })
  })

  it('can have wrapped functions access INPUT object', function (done) {
    var script = `function wrapped(){
      function add(first, second){
        return first + Number(INPUT.a);
      }
      return add(20, 22)
    }
     RESULT = wrapped()`

    sandbox(script, null, { INPUT: { a: 22 } }, function (err, res) {
      debug('err', err)
      debug('res', res)

      res.result.should.eql(42)
      done()
    })
  })

  // it('TODO: can have wrapped function using "done()" ', function(done) {
  //   // var myCode = "x=1;rp('http://www.google.com').then( function(){x=2 \n callback() })"
  //   var myCode = "function botFunction(){\
  //     a=1; setTimeout(function(){\
  //       a=2;\
  //       return a\
  //     }, 10);\
  //     return a\
  //   };\
  //     RESULT = botFunction() "
  //
  //   myLibraries = []
  //   sandbox( myCode , myLibraries, null, function(err, res ){
  //     debug('res: ', res);
  //     res.result.should.eql(2);
  //     done();
  //   })
  // });

  it('done() async api works with long-running async functions', function (done) {
    var myCode = 'RESULT=1; setTimeout(function () { RESULT=2 \n done() }, 100)'
    // var myCode = "a=3 \n RESULT = 1+a \n done()"

    var myLibraries = [{ namespace: 't', npm: 'timeout' }]
    sandbox(myCode, myLibraries, null, function (err, res) {
      if (err) {
        debug('err', err)
      } else {
        debug('res', res)
      }
      res.result.should.eql(2)
      done()
    })
  })

  it('done() can return a value async', function (done) {
    var myCode = 'RESULT=1; setTimeout(function(){ done( 2 ) }, 100)'
    // var myCode = "a=3 \n RESULT = 1+a \n done()"

    var myLibraries = [{ namespace: 't', npm: 'timeout' }]
    sandbox(myCode, myLibraries, null, function (err, res) {
      if (err) {
        debug('err', err)
      } else {
        debug('res', res)
      }
      res.result.should.eql(2)
      done()
    })
  })

  it('double done() only returns first result', function (done) {
    var myCode = `RESULT=1
    setTimeout(function(){
      done(2)
      done(3)
    }, 100)`
    // var myCode = "a=3 \n RESULT = 1+a \n done()"

    var myLibraries = [{ namespace: 't', npm: 'timeout' }]
    sandbox(myCode, myLibraries, null, function (err, res) {
      if (err) {
        debug('err', err)
      } else {
        debug('res', res)
      }
      res.result.should.eql(2)
      done()
    })
  })

  // it('should run async libraries like request-promise', function(done) {
  //   var myCode = `rp('http://mockbin.org/bin/9ff60ad5-8adb-4d0e-b3a8-4952b87a6888')
  //   .then( function ( htmlBody ) {
  //     debug( 'htmlBody', htmlBody);
  //     done(htmlBody)
  //   }).catch( function(err) {
  //     debug('err', err )
  //   })`
  //
  //   myLibraries = [{ namespace: 't', npm: 'timeout'}]
  //
  //   sandbox( myCode , myLibraries, null, function(err, res ){
  //     result = JSON.parse( res.result )
  //     // debug('res', res);
  //     result.foo.should.eql('hello world');
  //     done();
  //   })
  // });

  it('should run async libraries like timeout', function (done) {
    var myCode = `
      t.timeout(1000, function(){
        done( { a: 1})
      })
    `

    var myLibraries = [{ namespace: 't', npm: 'timeout' }]

    sandbox(myCode, myLibraries, null, function (err, res) {
      if (err) {
        debug('err', err)
      } else {
        debug('res', res)
      }

      res.result.a.should.eql(1)
      done()
    })
  })

  // it('uses in-house Request promise', function(done) {
  //   // myLibraries = [{ namespace: 't', npm: 'timeout'}]
  //
  //   var myCode = `httpRequest({ url: 'http://mockbin.org/bin/9ff60ad5-8adb-4d0e-b3a8-4952b87a6888' })
  //   .then( function ( htmlBody ) {
  //     console.log( 'htmlBody', htmlBody);
  //     done(htmlBody)
  //
  //   }).catch( function(err) {
  //     console.log('err', err )
  //   })`
  //
  //   sandbox( myCode , null, null, function(err, res ){
  //     debug('err', err);
  //     debug('res', res.result)
  //     result = JSON.parse( res.result )
  //     // debug('res', res);
  //     result.foo.should.eql('hello world');
  //     done();
  //   })
  // });

  it('uses in-house request library function externally', function (done) {
    // for individual testt
    const requestlib = require('request')

    var httpRequestFunction = function (requestObj) {
      return new Promise(function (resolve, reject) {
        requestlib(requestObj, function (err, IncomingMessage, body) {
          if (err) {
              // console.log('httpRequest err', err);
            return reject(err)
          }
          resolve(body)
        })
      })
    }

    var myCode = `httpRequestFunction({ url: 'http://mockbin.org/bin/9ff60ad5-8adb-4d0e-b3a8-4952b87a6888' })
      .then( function ( htmlBody ) {
        console.log( 'htmlBody', htmlBody);
        done(htmlBody)

      }).catch( function(err) {
        console.log('err', err )
      })`

    sandbox(myCode, null, { httpRequestFunction: httpRequestFunction }, function (err, res) {
      if (err) debug('err', err); else debug('res', res.result)

      var result = JSON.parse(res.result)
      // debug('res', res);
      result.foo.should.eql('hello world')
      done()
    })
  })

  it('uses native ES6 Promise in sandbox', function (done) {
    // myLibraries = [{ namespace: 't', npm: 'timeout'}]
    var debugPromise = function (requestObj) {
      return new Promise(function (resolve, reject) {
        resolve({ a: 1 })
      })
    }

    var myCode = `debugPromise()
    .then( function ( res ) {
      console.log( 'res', res);
      done(res)

    }).catch( function(err) {
      console.log('err', err )
    })`

    sandbox(myCode, null, { debugPromise: debugPromise }, function (err, res) {
      if (err) {
        debug('err', err)
      } else {
        debug('res', res)
      }

      res.result.a.should.eql(1)
      done()
    })
  })

  // TODO:
  it('Unknown errors reported ', function (done) {
    var myCode = 'function test(){ return INPUT[0] }; RESULT = test()'

    sandbox(myCode, null, { INPUT: { a: 22 } }, function (err, res) {
      debug('res', res)
      debug('err', err)

      // result = JSON.parse( res.result )
      // debug('parsed res', result);
      res.status.should.eql('COMPLETE')
      done()
    })
  })

  it('Allows variable assignment within Request', function (done) {
    // for individual testt
    const requestlib = require('request')

    var httpRequestFunction = function (requestObj) {
      return new Promise(function (resolve, reject) {
        requestlib(requestObj, function (err, IncomingMessage, body) {
          if (err) {
              // console.log('httpRequest err', err);
            return reject(err)
          }
          resolve(body)
        })
      })
    }

    var myCode = `var assignment = 'useless';
    httpRequestFunction({ url: 'http://mockbin.org/bin/9ff60ad5-8adb-4d0e-b3a8-4952b87a6888' })
    .then(function( body ){
      done( body );
    })`

    sandbox(myCode, null, { httpRequestFunction: httpRequestFunction }, function (err, res) {
      if (err) {
        debug('err', err)
      } else {
        debug('res', res)
      }

      var result = JSON.parse(res.result)
      debug('parsed res', result)
      result.foo.should.eql('hello world')
      done()
    })
  })

  it('returns undefined library correctly', function (done) {
    var script = `function wrapped(){
        return request({ url: "http://mockbin.org/bin/17889536-56e3-498d-9caa-be279a7e5a8d/" })
        .then(function( body ){ RESULT = body; done()})
      };
      wrapped()`

    sandbox(script, null, { INPUT: { a: 22 } }, function (err, res) {
      debug('sandbox2test.js - err', err)
      debug('sandbox2test.js - res', res)
      err.status.should.eql('ERROR')
      done()
    })
  })

  it('TODO: issue identifying async done vs return', function (done) {
    this.skip()
    var myCode = `
      RESULT=1
      if (false) {
        setTimeout(function () {
           RESULT=2
           done()
         }, 100)
      } else {
        RESULT=2
      }`
      // var myCode = "a=3 \n RESULT = 1+a \n done()"

    myLibraries = [{ namespace: 't', npm: 'timeout'}]
    sandbox(myCode, myLibraries, null, function (err, res) {
      res.result.should.eql(2)
      done()
    })
  })

  it('uses native ES6 async/await syntax', function (done) {
    this.timeout(5 * 1000)
    // myLibraries = [{ namespace: 't', npm: 'timeout'}]
    var debugPromise = function (requestObj) {
      return new Promise(function (resolve, reject) {
        resolve({ a: 1 })
      })
    }

    var myCode = `
    async function test () {
      var a = await debugPromise()
      console.log(a)
      done(a)
      return a
    }
    test()
    `

    sandbox(myCode, null, { debugPromise: debugPromise }, function (err, res) {
      if (err) {
        debug('err', err)
      } else {
        debug('res', res)
      }

      res.result.a.should.eql(1)
      done()
    })
  })

  it('uses native ES6 let const', function (done) {
    // myLibraries = [{ namespace: 't', npm: 'timeout'}]
    var debugPromise = function (requestObj) {
      return new Promise(function (resolve, reject) {
        resolve({ a: 1 })
      })
    }

    var myCode = `
      function test () {
        let a = {a: 1}
        done(a)
      }
      test()
    `

    sandbox(myCode, null, { debugPromise: debugPromise }, function (err, res) {
      if (err) {
        debug('err', err)
      } else {
        debug('res', res)
      }

      res.result.a.should.eql(1)
      done()
    })
  })
})
